#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2027-03-20
# 
# collect/show Cisco hardware contract status
#
# 2017-05-29: fixed empty pid handling
#             added serial number cleanup
# 2018-04-09: removed import
# 2018-09-04: changes for CMK 1.5.x (inv_tree --> inv_tree_list)
# 2018-09-05: changes for CMK 1.5.x (replaced global variable g_hostname with api call host_name())
# 2019-08-13: chaged variable host_anme to _hostname for cmk1.5
# 2020-08-04: moved node tree from hardware.system.support to hardware.system
#             cleanup create_sn2info_record
# 2021-07-24: rewritten for CMK 2.0
# 2021-07-25: some cleanup
# 2021-07-30: moved parse function to utils
#
# ToDo: exit codes ??
#
import os
import json
import time

from typing import Dict

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    SNMPTree,
    contains,
    any_of,
    startswith,
    TableRow,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    InventoryResult,
)

from cmk.base.plugins.agent_based.utils.inv_cisco_support import (
    expand_path,
    pid_on_black_list,
    set_pid_black_list,
    get_base_path,
    parse_inv_cisco_contract_eox,
    SnmpContractEntry,
)


def _create_sn2info_record(sn2infofile, optional_columns):
    sn2info = {}
    if os.path.isfile(sn2infofile):
        with open(sn2infofile) as f:
            try:
                sn2inforecord = json.load(f)
            except ValueError as e:
                return {'serial_number': f'json load error {e}'}

        modifytime = os.path.getmtime(sn2infofile)
        sn2info.update({'Last_checked': time.strftime('%Y-%m-%d', time.localtime(modifytime))})

        # add nonempty values to sn2info record
        for key, value in [
            ('serial_number', sn2inforecord.get('sr_no')),
            ('is_covered', sn2inforecord.get('is_covered')),
            ('contract_site_customer_name', sn2inforecord.get('contract_site_customer_name')),
            ('contract_site_address1', sn2inforecord.get('contract_site_address1')),
            ('contract_site_city', sn2inforecord.get('contract_site_city')),
            ('contract_site_state_province', sn2inforecord.get('contract_site_state_province')),
            ('contract_site_country', sn2inforecord.get('contract_site_country')),
            ('service_line_descr', sn2inforecord.get('service_line_descr')),
            ('service_contract_number', sn2inforecord.get('service_contract_number')),
            ('covered_product_line_end_date', sn2inforecord.get('covered_product_line_end_date')),
            ('parent_sr_no', sn2inforecord.get('parent_sr_no')),
            ('warranty_type', sn2inforecord.get('warranty_type')),
            ('warranty_type_description', sn2inforecord.get('warranty_type_description')),
            ('warranty_end_date', sn2inforecord.get('warranty_end_date')),
        ]:
            value = value.strip()
            if value != '' and key not in optional_columns:
                sn2info[key] = value

    return sn2info


def inventory_cisco_contract(params, section: Dict[str, SnmpContractEntry]) -> InventoryResult:
    optionalcolumns = [
        'contract_site_state_province',
        'warranty_type_description',
        'warranty_end_date',
        'warranty_type',
    ]

    # get parameters from wato
    if params:
        optionalcolumns = params.get('removecolumns', optionalcolumns)
        set_pid_black_list(params.get('PID_black_list'))

    base_path = get_base_path()
    sn2info_dir = base_path + '/sn2info'
    path_found = expand_path(sn2info_dir + '/found/')
    path_not_found = expand_path(sn2info_dir + '/not_found/')
    path_request = expand_path(sn2info_dir + '/request/')

    path = ['hardware', 'support', 'cisco_contract']

    for serial in section.keys():
        if not pid_on_black_list(section[serial].pid):
            key_columns = {'serial_number': serial}
            serialscontract = ({
                'pid': section[serial].pid,
                'ProductIDDescription': section[serial].phydescr,
                'is_covered': 'requested',
            })
            # find serial in SER-sn2info-covered
            if os.path.isfile(path_found + serial):
                serialscontract.update(_create_sn2info_record(path_found + serial, optionalcolumns))
                serialscontract.pop('serial_number')
            # find serial in SER-sn2info-notcovered
            elif os.path.isfile(path_not_found + serial):
                modifytime = os.path.getmtime(path_not_found + serial)
                serialscontract.update({
                    'is_covered': 'NO',
                    'Last_checked': time.strftime('%Y-%m-%d', time.localtime(modifytime)),
                })
            else:
                # create new request in SER-sn2info-request
                if not os.path.isfile(path_request + serial):
                    with open(path_request + serial, 'w+') as f:  # only create file (touch)
                        pass
                modifytime = os.path.getmtime(path_request + serial)
                serialscontract.update({
                    'Last_checked': time.strftime('%Y-%m-%d', time.localtime(modifytime)),
                })
            yield TableRow(
                path=path,
                key_columns=key_columns,
                inventory_columns=serialscontract
            )


register.snmp_section(
    name='inv_cisco_contract',
    parse_function=parse_inv_cisco_contract_eox,
    fetch=SNMPTree(
        base='.1.3.6.1.2.1.47.1.1.1.1',  # ENTITY-MIB::entPhysicalEntry
        oids=[
            '2',  # entPhysicalDescr
            '11',  # entPhysicalSerialNum
            '13',  # entPhysicalModelName
        ]
    ),
    detect=any_of(
        contains('.1.3.6.1.2.1.1.1.0', 'cisco'),
        startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.9.1')
    )
)

register.inventory_plugin(
    name='inv_cisco_contract',
    inventory_function=inventory_cisco_contract,
    inventory_default_parameters={},
    inventory_ruleset_name='inv_cisco_contract',
)
