#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# License: GNU General Public License v2
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2017-03-20
#
# collect/show Cisco hardware EoX status
#
# 2017-05-29: fixed empty pid handling
#             added serial number cleanup
# 2018-04-09: removed import
# 2018-09-04: changes for CMK 1.5.x (inv_tree --> inv_tree_list)
# 2018-09-05: changes for CMK 1.5.x (replaced global variable g_hostname with api call host_name())
# 2018-08-13: changed variable host_name to _hostname for cmk 1.5
# 2020-08-04: code cleanup (_create_eox_record function)
#             moved node tree from hardware.system.support to hardware.system
# 2021-07-23: rewrite for CMK 2.0
# 2021-07-26: cleanup
# 2021-07-30: moved parse function to utils

import os
import json
import time

from typing import Dict

from cmk.base.plugins.agent_based.agent_based_api.v1 import (
    register,
    SNMPTree,
    contains,
    any_of,
    startswith,
    TableRow,
)
from cmk.base.plugins.agent_based.agent_based_api.v1.type_defs import (
    InventoryResult,
)

from cmk.base.plugins.agent_based.utils.inv_cisco_support import (
    expand_path,
    pid_on_black_list,
    sn_on_black_list,
    pid_on_bad_list,
    set_pid_bad_list,
    set_sn_black_list,
    get_base_path,
    parse_inv_cisco_contract_eox,
    SnmpContractEntry,
)


def _create_eox_record(eoxfile, optional_columns):
    eox = {}
    if os.path.isfile(eoxfile):
        with open(eoxfile) as f:
            try:
                eoxrecord = json.load(f)
            except ValueError:
                return {}

        modifytime = os.path.getmtime(eoxfile)

        eox['Last_checked'] = time.strftime('%Y-%m-%d', time.localtime(modifytime))
        eox['pid'] = eoxrecord.get('EOLProductID')

        # Migration Info
        eox_migration_details = eoxrecord.get('EOXMigrationDetails')

        for key, value in [
            ('ProductIDDescription', eoxrecord.get('ProductIDDescription', '')),
            ('EOXExternalAnnouncementDate', eoxrecord.get('EOXExternalAnnouncementDate').get('value', '')),
            ('EndOfSaleDate', eoxrecord.get('EndOfSaleDate').get('value', '')),
            ('EndOfSvcAttachDate', eoxrecord.get('EndOfSvcAttachDate').get('value', '')),
            ('LastDateOfSupport', eoxrecord.get('LastDateOfSupport').get('value', '')),
            ('ProductBulletinNumber', eoxrecord.get('ProductBulletinNumber', '')),
            ('LinkToProductBulletinURL', eoxrecord.get('LinkToProductBulletinURL', '')),
            ('UpdatedTimeStamp', eoxrecord.get('UpdatedTimeStamp').get('value', '')),
            ('EndOfSecurityVulSupportDate', eoxrecord.get('EndOfSecurityVulSupportDate').get('value', '')),
            ('EndOfSWMaintenanceReleases', eoxrecord.get('EndOfSWMaintenanceReleases').get('value', '')),
            ('EndOfRoutineFailureAnalysisDate', eoxrecord.get('EndOfRoutineFailureAnalysisDate').get('value', '')),

            # Migration Info
            ('MigrationProductId', eox_migration_details.get('MigrationProductId', '')),
            ('MigrationInformation', eox_migration_details.get('MigrationInformation', '')),
            ('MigrationProductInfoURL', eox_migration_details.get('MigrationProductInfoURL', '')),
            ('MigrationProductName', eox_migration_details.get('MigrationProductName', '')),
        ]:
            value = value.strip()
            if value != '' and key not in optional_columns:
                eox[key] = value

        # add serial if EoX from get_eox_by_serial
        eox_input_type = eoxrecord.get('EOXInputType')
        if eox_input_type == 'ShowEOXBySerialNumber':
            eox['serial_number'] = eoxrecord.get('EOXInputValue')

    return eox


def inventory_cisco_eox(params, section: Dict[str, SnmpContractEntry]) -> InventoryResult:
    always_use_serial = False
    optionalcolumns = ['EndOfSecurityVulSupportDate',
                       'EndOfSWMaintenanceReleases',
                       'EndOfRoutineFailureAnalysisDate',
                       'MigrationInformation',
                       'MigrationProductInfoURL',
                       'MigrationProductName',
                       ]

    # get parameters from wato
    if params:
        always_use_serial = params.get('always_use_serial', always_use_serial)
        optionalcolumns = params.get('removecolumns', optionalcolumns)
        set_sn_black_list(params.get('SN_black_list'))
        set_sn_black_list(params.get('PID_black_list'))
        set_pid_bad_list(params.get('PID_bad_list'))

    base_path = get_base_path()
    eox_path = base_path + '/EoX'
    path_found = eox_path + '/found'
    path_not_found = eox_path + '/not_found'
    path_request = eox_path + '/request'

    path_request_pid = expand_path(path_request + '/pid')
    path_found_pid = expand_path(path_found + '/pid')
    path_not_found_pid = expand_path(path_not_found + '/pid')

    path_request_ser = expand_path(path_request + '/ser')
    path_found_ser = expand_path(path_found + '/ser')
    path_not_found_ser = expand_path(path_not_found + '/ser')

    path = ['hardware', 'support', 'cisco_eox']

    pids = []
    pidseox = {}
    serials = []
    serialseox = {}
    nodes = {}

    for serial in section.keys():
        # drop all PIDs/SNs on Black List
        if not pid_on_black_list(section[serial].pid) and not sn_on_black_list(serial):
            if section[serial].phydescr == '':
                section[serial].phydescr = 'requested'

            # check if PID on bad list, if so try serial number
            if pid_on_bad_list(section[serial].pid) or always_use_serial:
                serials.append(serial)
            else:
                pids.append(section[serial].pid)

            # create default node entry for PID/SN
            key_columns = {'serial_number': serial}
            yield TableRow(
                path=path,
                key_columns=key_columns,
                inventory_columns={
                    'pid': section[serial].pid,
                    'ProductIDDescription': section[serial].phydescr,
                }
            )
            nodes[serial] = {
                'pid': section[serial].pid,
                'ProductIDDescription': section[serial].phydescr,
            }

    # remove duplicates
    pids = list(set(pids))
    for pid in pids:
        pidfile = pid.replace('/', '_')
        # find pid in PID-EOX-known
        if os.path.isfile(path_found_pid + pidfile):
            pidseox.update({pid: _create_eox_record(path_found_pid + pidfile, optionalcolumns)})
        # find pid in PID-EOX-unknown
        elif os.path.isfile(path_not_found_pid + pidfile):
            modifytime = os.path.getmtime(path_not_found_pid + pidfile)
            pidseox.update({pid: {
                'pid': pid,
                'EOXExternalAnnouncementDate': 'None found',
                'Last_checked': time.strftime('%Y-%m-%d', time.localtime(modifytime)),
            }})
        else:
            # create new request in PID-EOX-request
            if not os.path.isfile(path_request_pid + pidfile):
                with open(path_request_pid + pidfile, 'w+') as f:  # only create file
                    # only create file
                    pass
            modifytime = os.path.getmtime(path_request_pid + pidfile)
            pidseox.update({pid: {
                'pid': pid,
                'EOXExternalAnnouncementDate': 'requested',
                'Last_checked': time.strftime('%Y-%m-%d', time.localtime(modifytime)),
            }})

    for serial in serials:
        # find serial in SER-EOX-known
        if os.path.isfile(path_found_ser + serial):
            serialseox.update({serial: _create_eox_record(path_found_ser + serial, optionalcolumns)})
        # find serial in SER-EOX-unknown
        elif os.path.isfile(path_not_found_ser + serial):
            modifytime = os.path.getmtime(path_not_found_ser + serial)
            serialseox.update({serial: {
                'serial_number': serial,
                'EOXExternalAnnouncementDate': 'None found',
                'Last_checked': time.strftime('%Y-%m-%d', time.localtime(modifytime))
            }})
        else:
            # create new request in SER-EOX-request
            if not os.path.isfile(path_request_ser + serial):
                with open(path_request_ser + serial, 'w+') as f:  # only create file
                    pass
            modifytime = os.path.getmtime(path_request_ser + serial)
            serialseox.update({serial: {
                'serial_number': serial,
                'EOXExternalAnnouncementDate': 'requested',
                'Last_checked': time.strftime('%Y-%m-%d', time.localtime(modifytime)),
            }})

    for physerialnum in nodes.keys():
        # create default node entry for PID/SN
        key_columns = {'serial_number': physerialnum}
        if nodes[physerialnum]['pid'] in pidseox.keys():
            yield TableRow(
                path=path,
                key_columns=key_columns,
                inventory_columns=pidseox[nodes[physerialnum]['pid']]
            )
        if physerialnum in serialseox.keys():
            serialseox[physerialnum].pop('serial_number')
            yield TableRow(
                path=path,
                key_columns=key_columns,
                inventory_columns=serialseox[physerialnum]
            )


register.snmp_section(
    name='inv_cisco_eox',
    parse_function=parse_inv_cisco_contract_eox,
    fetch=SNMPTree(
        base='.1.3.6.1.2.1.47.1.1.1.1',  # ENTITY-MIB::entPhysicalEntry
        oids=[
            '2',  # entPhysicalDescr
            '11',  # entPhysicalSerialNum
            '13',  # entPhysicalModelName
        ]
    ),

    detect=any_of(
        contains('.1.3.6.1.2.1.1.1.0', 'cisco'),
        startswith('.1.3.6.1.2.1.1.2.0', '.1.3.6.1.4.1.9.1')
    )
)

register.inventory_plugin(
    name='inv_cisco_eox',
    inventory_function=inventory_cisco_eox,
    inventory_default_parameters={},
    inventory_ruleset_name='inv_cisco_eox',
)
