#!/usr/bin/env python3
# -*- coding: utf-8 -*-
#
# Author: thl-cmk[at]outlook[dot]com
# URL   : https://thl-cmk.hopto.org
# Date  : 2018-01-25
#
# add on for cisco support api. calls cisco bug api 2.0.
# for more information see: https://developer.cisco.com/docs/support-apis/#bug
#
# 2021-07-24: rewrite for python3.8
#
import os
import json

from cisco_live_cycle_utils import (
    configure_logger,
    log_message,
    expand_path,
    get_ids_from_dir,
    get_subdirs_from_dir,
    remove_empty_sub_dirs,
    sleep_random,
    move_dir,
)
from ciscoapi import (
    AccessToken,
    Settings,
    get_bug_by_pid_and_release,
)


def main():
    settings = Settings()
    access_token = AccessToken(settings.client_id, settings.client_secret, settings.proxies)
    configure_logger(log_level=settings.log_level)

    bug_path = settings.base_path + '/bug'
    path_found = expand_path(bug_path + '/found')
    path_not_found = expand_path(bug_path + '/not_found')
    path_request = expand_path(bug_path + '/request')
    path_missing = expand_path(bug_path + '/missing')

    pids_requested = {}
    pids_refresh = {}

    # get list of bug reports to refresh
    pids = get_subdirs_from_dir(path_found)
    for pid in pids:
        pids_refresh[pid.replace('_', '/')] = ','.join(get_ids_from_dir(path_found+pid,
                                                                        refresh_time=settings.bug_refresh_found))

    # move not_found bug reports older then 'refresh_notfound' days to request (for refresh)
    move_dir(path_not_found, path_request, refresh_time=settings.bug_refresh_not_found)

    # get list of PIDs requests
    pids = get_subdirs_from_dir(path_request)
    log_message(f'bug requests (PIDs): {pids}')
    for pid in pids:
        pids_requested[pid.replace('_', '/')] = ','.join(get_ids_from_dir(path_request+pid))
    log_message(f'bug requests (PIDs and releases): {pids_requested}')

    if len(pids_refresh.keys()) > 0 or len(pids_requested.keys()) > 0:

        reqoptions = []

        # modified_date
        # 1 = Last Week
        # 2 = Last 30 Days(default)
        # 3 = Last 6  Months
        # 4 = Last Year
        # 5 = All
        reqoptions.append('modified_date=1')
#        reqoptions.append('modified_date=3')  # for testing

        # severity (default is all)
        # 1 = Severity 1
        # 2 = Severity 2
        # 3 = Severity 3
        # 4 = Severity 4
        # 5 = Severity 5
        # 6 = Severity 6
        # reqoptions.append('severity=2')

        # status (default is all)
        # O = Open
        # F = Fixed
        # T = Terminated
        # reqoptions.append('status=O')

        # sort_by
        #  status
        #  modified_date (recent first, default)
        #  severity
        #  support_case_count
        #  modified_date_earliest (earliest first)
        # reqoptions.append('sort_by=severity')

        # reqoptions.append('page_index=2')

        if len(reqoptions) > 0:
            reqoptions = '?' + '&'.join(reqoptions)
        else:
            reqoptions = ''

        # wait random time after startup (load spread)
        if settings.wait_after_start:
            sleep_random(settings.max_wait_time)

        # first refresh bug reports
        for pid in pids_refresh.keys():

            # get bug records for time frame
            bug_records = get_bug_by_pid_and_release(
                pid,
                pids_refresh.get(pid),
                access_token,
                reqoptions,
                settings=settings
            )
            for entry in bug_records:
                pid = entry.get('pid')
                software_releases = entry.get('software_releases')
                status_code = int(entry.get('status_code', 200))
                log_message(f'bug return:PID: {pid}, Status: {status_code}, Version: {software_releases}')
                path = expand_path(path_found + pid.replace('/', '_'))
                # check if there were was an error, if so go to next pid
                if status_code == 200:
                    bugs = entry.get('bugs', None)
                    if bugs:  # if new/changed bugs found
                        for bug in bugs:
                            bug.pop('description')  # remove description
                        for release in software_releases.keys():  # create one bug list per software release
                            bug_release = software_releases.get(release)
                            log_message(f'bug found:PID: {pid}, Releases: {release}')
                            new_bugs = []
                            for bug in bugs:
                                if bug_release in bug.get('known_affected_releases'):
                                    new_bugs.append(bug)

                            # open found file
                            with open(path + release) as f:
                                try:
                                    bug_record = json.load(f)
                                except ValueError as e:
                                    log_message(f'{pid}:{release}:snmp_cisco_bug:bug_found:JSON load error: {e}',
                                                level='WARNING')

                            found_bugs = bug_record.get('bugs')
                            for found_bug in found_bugs:
                                for new_bug in new_bugs:
                                    if found_bug.get('bug_id') == new_bug.get('bug_id'):
                                        found_bug.update(new_bug)  # update old bug with new data (changed bugs)
                                        new_bugs.remove(new_bug)
                            for bug in new_bugs:
                                found_bugs.append(bug)  # add new bugs
                            bug_record['bugs'] = found_bugs  # replace bug list with new list
                            bug_record['total_records'] = len(found_bugs)  # change number of records

                            with open(path + release, 'w') as f:
                                json.dump(bug_record, f)  # write updated bug report

                    else:
                        for release in software_releases.keys():
                            if os.path.exists(path + release):
                                os.utime(path + release, None)

        reqoptions = []

        # modified_date
        # 1 = Last Week
        # 2 = Last 30 Days(default)
        # 3 = Last 6  Months
        # 4 = Last Year
        # 5 = All
        reqoptions.append('modified_date=5')
#        reqoptions.append('modified_date=2')  # for testing

        # severity (default is all)
        # 1 = Severity 1
        # 2 = Severity 2
        # 3 = Severity 3
        # 4 = Severity 4
        # 5 = Severity 5
        # 6 = Severity 6
        # reqoptions.append('severity=2')

        # status (default is all)
        # O = Open
        # F = Fixed
        # T = Terminated
        # reqoptions.append('status=O')

        # sort_by
        #  status
        #  modified_date (recent first, default)
        #  severity
        #  support_case_count
        #  modified_date_earliest (earliest first)
        # reqoptions.append('sort_by=severity')

        # reqoptions.append('page_index=2')

        if len(reqoptions) > 0:
            reqoptions = '?' + '&'.join(reqoptions)
        else:
            reqoptions = ''

        for pid in pids_requested.keys():
            bug_records = get_bug_by_pid_and_release(
                pid,
                pids_requested.get(pid),
                access_token,
                reqoptions,
                settings=settings
            )
            for entry in bug_records:
                pid = entry.get('pid')
                software_releases = entry.get('software_releases')
                status_code = int(entry.get('status_code', 200))
                log_message(f'bug return:PID: {pid}, Status: {status_code}, Version: {software_releases}')
                # check if there where was an error, if so go to next pid
                if status_code == 200:
                    bugs = entry.get('bugs', None)
                    if bugs:
                        for bug in bugs:
                            bug.pop('description')
                    for release in software_releases.keys():  # create one bug list per software release
                        bug_release = software_releases.get(release)
                        if bugs:
                            log_message(f'bug found:PID: %s{pid}, Releases: {release}')
                            missing = entry.get('missing', {})
                            # split bugs by release
                            release_bugs = {
                                'pid': pid,
                                'software_release': release,
                                'bugs': [],
                                'missing': missing
                            }
                            for bug in bugs:
                                if bug_release in bug.get('known_affected_releases'):
                                    release_bugs['bugs'].append(bug)
                            release_bugs['total_records'] = len(release_bugs['bugs'])

                            path = expand_path(path_found + pid.replace('/', '_'))
                            with open(path + release, 'w') as f:
                                json.dump(release_bugs, f)

                            if len(missing.keys()) != 0:
                                path = expand_path(path_missing + pid.replace('/', '_'))
                                with open(path + release, 'w') as f:
                                    json.dump(missing, f)
                        else:
                            log_message(f'bug not found:PID: {pid}, Version: {release}')
                            path = expand_path(path_not_found + pid.replace('/', '_'))
                            log_message(f'not found: {entry}')
                            with open(path + release, 'w') as f:
                                json.dump(entry, f)
                                pass

                        # remove request file
                        try:
                            log_message(f'bug delete request:PID: {pid}, Version: {release}')
                            os.remove(path_request + pid.replace('/', '_') + '/' + release)
                        except OSError:
                            pass

        # clean up (remove empty directories)
        remove_empty_sub_dirs(path_request)
        remove_empty_sub_dirs(path_found)
        remove_empty_sub_dirs(path_not_found)
        remove_empty_sub_dirs(path_missing)


main()
